use godot::{
    classes::{IVideoStream, IVideoStreamPlayback, Texture2D, VideoStream, VideoStreamPlayback},
    prelude::*,
};
use pipewire::stream::Stream;

#[derive(GodotClass)]
#[class(base=VideoStream)]
pub(crate) struct PwVideoStreamNode {
    base: Base<VideoStream>,
    stream: Option<Stream>,
}

#[derive(GodotClass)]
#[class(base=VideoStreamPlayback)]
pub(crate) struct PwVideoStreamPlayback {
    base: Base<VideoStreamPlayback>,
    paused: bool,
}

#[godot_api]
impl IVideoStream for PwVideoStreamNode {
    fn init(base: Base<VideoStream>) -> Self {
        Self { base, stream: None }
    }
}

#[godot_api]
impl IVideoStreamPlayback for PwVideoStreamPlayback {
    fn init(base: Base<VideoStreamPlayback>) -> Self {
        Self { base, paused: true }
    }

    fn update(&mut self, _delta: f64) {
        if self.paused {
            return;
        }
        // TODO: Get latest frame or keep last one if no new one
        todo!()
    }

    fn play(&mut self) {
        self.paused = false;
        // TODO: Start copying frames again
        todo!()
    }

    fn stop(&mut self) {
        self.paused = true;
        // TODO: Stop updating the last known frame
        todo!()
    }

    fn is_paused(&self) -> bool {
        self.paused
    }

    fn set_paused(&mut self, paused: bool) {
        if paused {
            self.stop();
        } else {
            self.play();
        }
    }

    fn is_playing(&self) -> bool {
        !self.paused
    }

    fn get_texture(&self) -> Option<Gd<Texture2D>> {
        // TODO: Transform latest frame into Godot image
        todo!()
    }
}

impl PwVideoStreamPlayback {
    pub fn setup(&mut self) {
        // TODO: Store pipewire stream and fetch first frame
    }
}
